/* This source is placed in the public domain.  Do what you want with it */

#include <unistd.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <signal.h>
#include <asm/io.h>
#include <config.h>
#include <string.h>
#include <sys/types.h>
#include <string.h>

/* some those includes are for the serial port buffer check defs */


#define data_delimit '\n'
unsigned char portdata[10000];
int fdSer;
int n;
int m;
int limitm;
int limitn;

int main(int argc, unsigned char* argv[]) {

    static char version[] = "AOR serial port transceiver module, Emarit Ranu.  v1.0";
    struct termios termattribs;
    struct termios *defaults_serial;
    void outport(char);
    void usage();
    void readport();
    void cleanup();
    int buffer_wait();
    int port_open(); 
    signal(SIGINT, cleanup);

    /*  If we weren't give any arguments and/or data, exit with usage */
    if (argc == 1 || argc == 2) {
	usage(argv);
    }


    fdSer = port_open(argc, argv);   /* grab the port fd */
    if (fdSer == -1) {
        exit(3);
    }
    tcgetattr(fdSer,defaults_serial);      /* save current serial settings */

    termattribs.c_iflag = IGNBRK | IGNPAR | IMAXBEL | IXOFF;
    termattribs.c_oflag = 0;  
    termattribs.c_cflag = CS8 | CSTOPB | CREAD | CLOCAL;
    termattribs.c_lflag = 0;		    /* Set some term flags */
    
    /*  These ensure there are no read timeouts (possibly writes?) */
    termattribs.c_cc[VMIN] = 1; 
    termattribs.c_cc[VTIME] = 0;
        
    cfsetispeed(&termattribs,brate);	    /* Set input speed */
    cfsetospeed(&termattribs,brate);	    /* Set output speed */
    tcsetattr(fdSer,TCSANOW,&termattribs);  /* Set the serial port */

    /*  flush data on port */
    tcflush(fdSer,TCIOFLUSH);

/*
 * the following hunk of code properly formats the arguments to 'aor' to
 * be output on the serial port in proper order and includes spaces for
 * each arg into 'aor'.
 */

    limitn = argc;
    n = 2;

/* if the second arg is 'r' then we must call readport() */
    if (strncmp(argv[2],"r",1) == 0 ) { 
	n = 3;   
	}
    while (n != limitn) {
        m = 0;
 	limitm = strlen(argv[n]);
        while (m != limitm) {
/*                  printf("%c",argv[n][m]); */
 		outport(argv[n][m]);
                m = m + 1;
                }
        n = n + 1;
	outport(*" "); 
/*	printf(" ");  */
    }
 	outport(*"\r");
 	outport(data_delimit);
/*  	printf("\r\n");  */

    if (strncmp(argv[2],"r",1) == 0 ) { 
	readport();
	}
    exit(0);
}










/*
 *  This function returns the fd for the serial port.
 *  This needs to be rewritten to handle ANY serial port number via user
 *  argument.  Some people may have more than four serial ports.
 */

int port_open(int argc, char* argv[]) {
    int fdSer1;
    void usage();

    if (argc > 0) {
       switch(argv[1][0]) {
             case '0':
             if ((fdSer1 = open("/dev/ttyS0", O_RDWR )) < 0) {
                perror("open of /dev/ttyS0 failed!!!\n");
                exit(3);
             }
             break;

             case '1':
             if ((fdSer1 = open("/dev/ttyS1", O_RDWR )) < 0) {
                perror("open of /dev/ttyS1 failed!!!\n");
                exit(3);
             }
             break;

             case '2':
             if ((fdSer1 = open("/dev/ttyS2", O_RDWR )) < 0) {
                perror("open of /dev/ttyS2 failed");
                exit(3);
             }
             break;

             case '3':
             if ((fdSer1 = open("/dev/ttyS3", O_RDWR )) < 0) {
                perror("open of /dev/ttyS3 failed");
                exit(3);
             }
             break;
             default:
             printf("Invalid serial port number!!!\n  Remember:  com1 is 0, com2 is 1, etc.\n");
             return(-1);
       }
    } else {       /* no argument given, spit out usage */
	usage(argv);
    }
    return(fdSer1);
}



/* simple generic func to spew bytes out on the port */
void outport(char dataout) {
     write(fdSer,&dataout,1);
}




void readport() {
   int n,E;
   int buffer_wait();
   n = 0;
   while (n != 2001) {
	portdata[n] = *" ";
	n = n + 1;
   }
   n = 0;
   E = buffer_wait();
   while (E == 1) {
/*        printf("%d\n",buffer_wait(fdSer));  */
	read(fdSer,&portdata[n],1);
	printf("%c",portdata[n]);
	n = n + 1;
	E = buffer_wait();
   }
   printf("\n");
}


/* This function mearly determines if something is waiting to be read 
 * on the port
 */
/* You can use a function like this. It returns 1 if data is waiting or 0
 * if not.
 * Thanks to Michael 'Ghandi' Herold.
 */

int buffer_wait()
{
   struct timeval timeout;

   fd_set   fds;
   int      result;

   FD_ZERO(&fds);
   FD_SET(fdSer, &fds);

   timeout.tv_sec    = 0;  /* waits for char a max of 1 second */
   timeout.tv_usec   = waittime;
   result = select(FD_SETSIZE , &fds, NULL, NULL, &timeout);
   if (result > 0) return(1);

   return(0);
}








void usage(char* argv[])
{
printf("\nData output module, %s v1.0, by Emarit Ranu.\n",argv[0]);
printf("For use in conjunction with aor-tcl\n\n");
printf("usage:  aor [port #] [r] [data for port]\n\n");
exit(3);
}

void bye_msg() {
     /* generic message to spew when quit or killed */
printf("\n\nThe AOR-AR8000 control program in the making.\n\n");

}


void cleanup(int fdSer, struct termios *defaults_serial)
{
/* attempts to reset serial port to original state, closes fd
 * as I recall, this never reset the serial port.
 */
  tcsetattr(fdSer, TCSANOW, defaults_serial); 
  tcflush(fdSer,TCIOFLUSH);  /* flush IO data on port */
  close(fdSer);
  bye_msg();
  printf("Killed by signal 2!\n"); 
  exit(3);
}


